package project2

import org.scalatest._

class InterpretTest extends TimedSuite {
  import Language._

  def testInterpreter(ast: Exp, res: Int) = {
    val interpreter = new StackInterpreter

    assert(res == interpreter.run(ast), "Interpreter does not return the correct value")
  }

  test("arithm") {
    testInterpreter(Lit(-21), -21)
    testInterpreter(Unary("-", Lit(23)), -23)
    testInterpreter(Unary("+", Lit(22)), 22)
    testInterpreter(Prim("-", Lit(10), Lit(2)), 8)
    testInterpreter(Prim("+", Prim("+", Prim("-", Lit(1), Lit(2)), Lit(3)), Lit(4)), 6)
    testInterpreter(Prim("*", Lit(2), Lit(-3)), -6)
    testInterpreter(Prim("+", Prim("+", Prim("-", Unary("-", Lit(5)), Prim("/", Lit(8), Lit(3))),Lit(2)), Lit(3)), -2)
  }

  test("let&ref") {
    testInterpreter(Let("x", Lit(2), Ref("x")), 2) 
    testInterpreter(Let("x", Prim("*", Lit(2), Unary("-", Lit(5))), Ref("x")), -10)
  }

  test("if&cond") {
    testInterpreter(Let("x",Lit(5), Let("y", Lit(5), If(Cond("==", Ref("x"), Ref("y")),Lit(5), Lit(10)))), 5)
    testInterpreter(If(Cond(">", Lit(3), Lit(1)), Lit(10), Lit(5)), 10) 
    testInterpreter(If(Cond("==", Lit(1), Lit(2)), Lit(1), Lit(2)), 2)
    testInterpreter(VarDec("x",Lit(5),If(Cond("==",Ref("x"),Lit(5)),Lit(7),Lit(6))), 7)
    testInterpreter(VarDec("x",Lit(5),If(Cond("!=",Ref("x"),Lit(5)),Lit(7),Lit(6))), 6)
  }

  test("varassign") {
    testInterpreter(VarDec("x", Lit(5), VarAssign("x", Prim("-", Ref("x"), Lit(15)))), -10)
    testInterpreter(VarDec("x",Lit(5),VarAssign("x",Lit(3))), 3)
  }

  test("vardec") {
    testInterpreter(VarDec("x",Lit(5),Ref("x")), 5)
    testInterpreter(VarDec("x",Unary("-",Lit(5)),Ref("x")), -5)
  }

  test("while") {
    testInterpreter(While(Cond("!=",Lit(5),Lit(5)),Lit(3),Lit(5)), 5)
    testInterpreter(VarDec("x",Lit(2),VarDec("y",Lit(0),While(Cond("<",Ref("y"),Lit(3)),Let("dummy",VarAssign("x",Prim("*",Ref("x"),Ref("x"))),VarAssign("y",Prim("+",Ref("y"),Lit(1)))),Ref("x")))), 256)
    testInterpreter(VarDec("x",Lit(0),While(Cond("!=",Ref("x"),Lit(5)),VarAssign("x",Prim("+",Ref("x"),Lit(1))),Ref("x"))), 5)
  }
  
}
